@@ -45,26 +45,15 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     end
 
     def raise_nested_exceptions
-      raise_nested_exceptions_third
-    end
-
-    def raise_nested_exceptions_first
       raise "First error"
-    end
-
-    def raise_nested_exceptions_second
-      raise_nested_exceptions_first
     rescue
-      raise "Second error"
+      begin
+        raise "Second error"
+      rescue
+        raise "Third error"
+      end
     end
 
-    def raise_nested_exceptions_third
-      raise_nested_exceptions_second
-    rescue
-      raise "Third error"
-    end
-
-
     def call(env)
       env["action_dispatch.show_detailed_exceptions"] = @detailed
       req = ActionDispatch::Request.new(env)
@@ -86,8 +75,6 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
         raise ActionController::UnknownHttpMethod
       when "/not_implemented"
         raise ActionController::NotImplemented
-      when "/unprocessable_entity"
-        raise ActionController::InvalidAuthenticityToken
       when "/invalid_mimetype"
         raise ActionDispatch::Http::MimeNegotiation::InvalidType
       when "/not_found_original_exception"
@@ -183,6 +170,15 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     assert boomer.closed, "Expected to close the response body"
   end
 
+  test "returns empty body on HEAD cascade pass" do
+    @app = DevelopmentApp
+
+    head "/pass"
+
+    assert_response 404
+    assert_equal "", body
+  end
+
   test "displays routes in a table when a RoutingError occurs" do
     @app = DevelopmentApp
     get "/pass", headers: { "action_dispatch.show_exceptions" => :all }
@@ -627,6 +623,25 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
     assert_not_empty (output.rewind && output.read).lines
   end
 
+  test "logs exception causes" do
+    @app = DevelopmentApp
+
+    output = StringIO.new
+
+    env = { "action_dispatch.show_exceptions"       => :all,
+            "action_dispatch.logger"                => Logger.new(output),
+            "action_dispatch.log_rescued_responses" => true }
+
+    get "/nested_exceptions", headers: env
+    assert_response 500
+    log = output.rewind && output.read
+    assert_includes log, <<~MSG
+      Causes:
+      RuntimeError (Second error)
+      RuntimeError (First error)
+    MSG
+  end
+
   test "display backtrace when error type is SyntaxError" do
     @app = DevelopmentApp
 
@@ -744,28 +759,28 @@ class DebugExceptionsTest < ActionDispatch::IntegrationTest
         # Possible Ruby 3.4-dev bug: https://bugs.ruby-lang.org/issues/19117#note-45
         # assert application trace refers to line that raises the last exception
         assert_select "#Application-Trace-0" do
-          assert_select "code a:first", %r{in '.*raise_nested_exceptions_third'}
+          assert_select "code a:first", %r{in '.*raise_nested_exceptions'}
         end
 
         # assert the second application trace refers to the line that raises the second exception
         assert_select "#Application-Trace-1" do
-          assert_select "code a:first", %r{in '.*raise_nested_exceptions_second'}
+          assert_select "code a:first", %r{in '.*raise_nested_exceptions'}
         end
       else
         # assert application trace refers to line that raises the last exception
         assert_select "#Application-Trace-0" do
-          assert_select "code a:first", %r{in [`']rescue in .*raise_nested_exceptions_third'}
+          assert_select "code a:first", %r{in [`']rescue in rescue in .*raise_nested_exceptions'}
         end
 
         # assert the second application trace refers to the line that raises the second exception
         assert_select "#Application-Trace-1" do
-          assert_select "code a:first", %r{in [`']rescue in .*raise_nested_exceptions_second'}
+          assert_select "code a:first", %r{in [`']rescue in .*raise_nested_exceptions'}
         end
       end
 
       # assert the third application trace refers to the line that raises the first exception
       assert_select "#Application-Trace-2" do
-        assert_select "code a:first", %r{in [`'].*raise_nested_exceptions_first'}
+        assert_select "code a:first", %r{in [`'].*raise_nested_exceptions'}
       end
     end
   end
