@@ -8,9 +8,9 @@ module ActiveRecord
 
   module VERSION
     MAJOR = 7
-    MINOR = 1
-    TINY  = 5
-    PRE   = "1"
+    MINOR = 2
+    TINY  = 3
+    PRE   = nil
 
     STRING = [MAJOR, MINOR, TINY, PRE].compact.join(".")
   end
