@@ -5,7 +5,7 @@ require "cases/helper"
 class PostgreSQLPartitionsTest < ActiveRecord::PostgreSQLTestCase
   def setup
     @previous_unlogged_tables = ActiveRecord::ConnectionAdapters::PostgreSQLAdapter.create_unlogged_tables
-    @connection = ActiveRecord::Base.connection
+    @connection = ActiveRecord::Base.lease_connection
     ActiveRecord::ConnectionAdapters::PostgreSQLAdapter.create_unlogged_tables = false
   end
 
@@ -15,7 +15,7 @@ class PostgreSQLPartitionsTest < ActiveRecord::PostgreSQLTestCase
   end
 
   def test_partitions_table_exists
-    skip unless ActiveRecord::Base.connection.database_version >= 100000
+    skip unless ActiveRecord::Base.lease_connection.database_version >= 100000
     @connection.create_table :partitioned_events, force: true, id: false,
       options: "partition by range (issued_at)" do |t|
       t.timestamp :issued_at
