@@ -9,9 +9,11 @@ module ActiveRecord
       def test_sqlite_creates_directory
         Dir.mktmpdir do |dir|
           dir = Pathname.new(dir)
-          @conn = Base.sqlite3_connection database: dir.join("db/foo.sqlite3"),
-                               adapter: "sqlite3",
-                               timeout: 100
+          @conn = SQLite3Adapter.new(
+            database: dir.join("db/foo.sqlite3"),
+            adapter: "sqlite3",
+            timeout: 100,
+          )
           @conn.connect!
 
           assert Dir.exist? dir.join("db")
