@@ -4,21 +4,54 @@ require "cases/helper"
 require "support/ddl_helper"
 require "models/book"
 require "models/post"
-
-require "active_support/error_reporter/test_helper"
+require "timeout"
 
 class TrilogyAdapterTest < ActiveRecord::TrilogyTestCase
   setup do
-    @conn = ActiveRecord::Base.connection
+    @conn = ActiveRecord::Base.lease_connection
   end
 
   test "connection_error" do
     error = assert_raises ActiveRecord::ConnectionNotEstablished do
-      ActiveRecord::Base.trilogy_connection(host: "invalid", port: 12345).connect!
+      ActiveRecord::ConnectionAdapters::TrilogyAdapter.new(host: "invalid", port: 12345).connect!
     end
     assert_kind_of ActiveRecord::ConnectionAdapters::NullPool, error.connection_pool
   end
 
+  test "timeout in transaction doesnt query closed connection" do
+    assert_raises(Timeout::Error) do
+      Timeout.timeout(0.1) do
+        @conn.transaction do
+          @conn.execute("SELECT SLEEP(1)")
+        end
+      end
+    end
+  end
+
+  test "timeout in fixture set insertion doesnt query closed connection" do
+    fixtures = [
+      ["traffic_lights", [
+        { "location" => "US", "state" => ["NY"], "long_state" => ["a"] },
+      ]]
+    ] * 1000
+
+    assert_raises(Timeout::Error) do
+      Timeout.timeout(0.1) do
+        @conn.insert_fixtures_set(fixtures)
+      end
+    end
+  end
+
+  test "timeout without referential integrity doesnt query closed connection" do
+    assert_raises(Timeout::Error) do
+      Timeout.timeout(0.1) do
+        @conn.disable_referential_integrity do
+          @conn.execute("SELECT SLEEP(1)")
+        end
+      end
+    end
+  end
+
   test "#explain for one query" do
     explain = @conn.explain("select * from posts")
     assert_match %r(posts), explain
@@ -49,15 +82,15 @@ class TrilogyAdapterTest < ActiveRecord::TrilogyTestCase
   end
 
   test "#supports_comments? answers true" do
-    assert @conn.supports_comments?
+    assert_predicate @conn, :supports_comments?
   end
 
   test "#supports_comments_in_create? answers true" do
-    assert @conn.supports_comments_in_create?
+    assert_predicate @conn, :supports_comments_in_create?
   end
 
   test "#supports_savepoints? answers true" do
-    assert @conn.supports_savepoints?
+    assert_predicate @conn, :supports_savepoints?
   end
 
   test "#requires_reloading? answers false" do
@@ -93,7 +126,7 @@ class TrilogyAdapterTest < ActiveRecord::TrilogyTestCase
   end
 
   test "#active? answers true with connection" do
-    assert @conn.active?
+    assert_predicate @conn, :active?
   end
 
   test "#active? answers false with connection and exception" do
@@ -169,6 +202,8 @@ class TrilogyAdapterTest < ActiveRecord::TrilogyTestCase
     @conn.cache do
       event_fired = false
       subscription = ->(name, start, finish, id, payload) {
+        next if payload[:name] == "SCHEMA"
+
         event_fired = true
 
         # First, we test keys that are defined by default by the AbstractAdapter
@@ -200,6 +235,8 @@ class TrilogyAdapterTest < ActiveRecord::TrilogyTestCase
 
       event_fired = false
       subscription = ->(name, start, finish, id, payload) {
+        next if payload[:name] == "SCHEMA"
+
         event_fired = true
 
         # First, we test keys that are defined by default by the AbstractAdapter
@@ -341,7 +378,7 @@ class TrilogyAdapterTest < ActiveRecord::TrilogyTestCase
     ActiveRecord::Base.establish_connection(
       db_config.configuration_hash.merge("read_timeout" => 1)
     )
-    connection = ActiveRecord::Base.connection
+    connection = ActiveRecord::Base.lease_connection
 
     error = assert_raises(ActiveRecord::AdapterTimeout) do
       connection.execute("SELECT SLEEP(2)")
@@ -360,6 +397,38 @@ class TrilogyAdapterTest < ActiveRecord::TrilogyTestCase
     assert_includes error.message, "/var/invalid.sock"
   end
 
+  test "EPIPE raises ActiveRecord::ConnectionFailed" do
+    assert_raises(ActiveRecord::ConnectionFailed) do
+      @conn.raw_connection.stub(:query, -> (*) { raise Trilogy::SyscallError::EPIPE }) do
+        @conn.execute("SELECT 1")
+      end
+    end
+  end
+
+  test "ETIMEDOUT raises ActiveRecord::ConnectionFailed" do
+    assert_raises(ActiveRecord::ConnectionFailed) do
+      @conn.raw_connection.stub(:query, -> (*) { raise Trilogy::SyscallError::ETIMEDOUT }) do
+        @conn.execute("SELECT 1")
+      end
+    end
+  end
+
+  test "ECONNREFUSED raises ActiveRecord::ConnectionFailed" do
+    assert_raises(ActiveRecord::ConnectionFailed) do
+      @conn.raw_connection.stub(:query, -> (*) { raise Trilogy::SyscallError::ECONNREFUSED }) do
+        @conn.execute("SELECT 1")
+      end
+    end
+  end
+
+  test "ECONNRESET raises ActiveRecord::ConnectionFailed" do
+    assert_raises(ActiveRecord::ConnectionFailed) do
+      @conn.raw_connection.stub(:query, -> (*) { raise Trilogy::SyscallError::ECONNRESET }) do
+        @conn.execute("SELECT 1")
+      end
+    end
+  end
+
   test "setting prepared_statements to true raises" do
     assert_raises ArgumentError do
       ActiveRecord::ConnectionAdapters::TrilogyAdapter.new(prepared_statements: true).connect!
@@ -371,14 +440,13 @@ class TrilogyAdapterTest < ActiveRecord::TrilogyTestCase
   def assert_notification(notification, expected_payload = {}, &block)
     notification_sent = false
 
-    subscription = lambda do |*args|
+    subscription = lambda do |_, _, _, _, payload|
       notification_sent = true
-      event = ActiveSupport::Notifications::Event.new(*args)
 
       expected_payload.each do |key, value|
         assert(
-          value === event.payload[key],
-          "Expected notification payload[:#{key}] to match #{value.inspect}, but got #{event.payload[key].inspect}."
+          value === payload[key],
+          "Expected notification payload[:#{key}] to match #{value.inspect}, but got #{payload[key].inspect}."
         )
       end
     end
