@@ -1,6 +1,7 @@
 # frozen_string_literal: true
 
 require "cases/helper"
+require "models/attachment"
 require "models/developer"
 require "models/project"
 require "models/company"
@@ -31,6 +32,9 @@ require "models/tree"
 require "models/node"
 require "models/club"
 require "models/cpk"
+require "models/sharded/blog"
+require "models/sharded/blog_post"
+require "models/sharded/comment"
 
 class BelongsToAssociationsTest < ActiveRecord::TestCase
   fixtures :accounts, :companies, :developers, :projects, :topics,
@@ -40,7 +44,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
   def test_belongs_to
     client = Client.find(3)
     first_firm = companies(:first_firm)
-    assert_sql(/LIMIT|ROWNUM <=|FETCH FIRST/) do
+    assert_queries_match(/LIMIT|ROWNUM <=|FETCH FIRST/) do
       assert_equal first_firm, client.firm
       assert_equal first_firm.name, client.firm.name
     end
@@ -103,10 +107,6 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     if current_adapter?(:Mysql2Adapter, :TrilogyAdapter)
       assert_no_match(/`firm_with_primary_keys_companies`\.`id`/, sql)
       assert_match(/`firm_with_primary_keys_companies`\.`name`/, sql)
-    elsif current_adapter?(:OracleAdapter)
-      # on Oracle aliases are truncated to 30 characters and are quoted in uppercase
-      assert_no_match(/"firm_with_primary_keys_compani"\."id"/i, sql)
-      assert_match(/"firm_with_primary_keys_compani"\."name"/i, sql)
     else
       assert_no_match(/"firm_with_primary_keys_companies"\."id"/, sql)
       assert_match(/"firm_with_primary_keys_companies"\."name"/, sql)
@@ -252,9 +252,9 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_equal 0, counter
     comment = comments.first
     assert_equal 0, counter
-    sql = capture_sql { comment.post }
+    queries = capture_sql_and_binds { comment.post }
     comment.reload
-    assert_not_equal sql, capture_sql { comment.post }
+    assert_not_equal queries, capture_sql_and_binds { comment.post }
   end
 
   def test_proxy_assignment
@@ -270,7 +270,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
   def test_raises_type_mismatch_with_namespaced_class
     assert_nil defined?(Region), "This test requires that there is no top-level Region class"
 
-    ActiveRecord::Base.connection.instance_eval do
+    ActiveRecord::Base.lease_connection.instance_eval do
       create_table(:admin_regions, force: true) { |t| t.string :name }
       add_column :admin_users, :region_id, :integer
     end
@@ -285,7 +285,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     Admin.send :remove_const, "Region" if Admin.const_defined?("Region")
     Admin.send :remove_const, "RegionalUser" if Admin.const_defined?("RegionalUser")
 
-    ActiveRecord::Base.connection.instance_eval do
+    ActiveRecord::Base.lease_connection.instance_eval do
       remove_column :admin_users, :region_id if column_exists?(:admin_users, :region_id)
       drop_table :admin_regions, if_exists: true
     end
@@ -404,6 +404,32 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_equal 3, book.shop_id
   end
 
+  def test_clearing_optional_cpk_belongs_to_should_preserve_shared_pk
+    book = Cpk::Book.create!(id: [1, 2], title: "The Well-Grounded Rubyist")
+    chapter = Cpk::OptionalChapter.create!(id: [1, 2], book: book)
+
+    assert_equal book, chapter.book
+    assert_equal [1, 2], chapter.id
+
+    chapter.update!(book: nil)
+
+    assert_equal [1, 2], chapter.id
+    assert_nil chapter.book_id
+    assert_equal chapter.author_id, book.author_id
+  end
+
+  def test_should_reload_association_on_model_with_query_constraints_when_foreign_key_changes
+    blog = Sharded::Blog.create!
+    blog_post = Sharded::BlogPost.create!(blog: blog)
+    comment = Sharded::Comment.create!(blog: blog)
+
+    # Load the association once
+    comment.blog_post
+    comment.blog_post_id = blog_post.id
+
+    assert_equal blog_post, comment.blog_post
+  end
+
   def test_building_the_belonging_object_with_implicit_sti_base_class
     account = Account.new
     company = account.build_firm
@@ -463,7 +489,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     Company.where(id: odegy_account.firm_id).update_all(name: "ODEGY")
     assert_equal "Odegy", odegy_account.firm.name
 
-    assert_queries(1) { odegy_account.reload_firm }
+    assert_queries_count(1) { odegy_account.reload_firm }
 
     assert_no_queries { odegy_account.firm }
     assert_equal "ODEGY", odegy_account.firm.name
@@ -472,7 +498,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
   def test_reload_the_belonging_object_with_query_cache
     odegy_account_id = accounts(:odegy_account).id
 
-    connection = ActiveRecord::Base.connection
+    connection = ActiveRecord::Base.lease_connection
     connection.enable_query_cache!
     connection.clear_query_cache
 
@@ -485,12 +511,12 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_equal 2, connection.query_cache.size
 
     # Clear the cache and fetch the firm again, populating the cache with a query
-    assert_queries(1) { odegy_account.reload_firm }
+    assert_queries_count(1) { odegy_account.reload_firm }
 
     # This query is not cached anymore, so it should make a real SQL query
-    assert_queries(1) { Account.find(odegy_account_id) }
+    assert_queries_count(1) { Account.find(odegy_account_id) }
   ensure
-    ActiveRecord::Base.connection.disable_query_cache!
+    ActiveRecord::Base.lease_connection.disable_query_cache!
   end
 
   def test_resetting_the_association
@@ -501,7 +527,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_equal "Odegy", odegy_account.firm.name
 
     assert_no_queries { odegy_account.reset_firm }
-    assert_queries(1) { odegy_account.firm }
+    assert_queries_count(1) { odegy_account.firm }
     assert_equal "ODEGY", odegy_account.firm.name
   end
 
@@ -729,7 +755,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
   def test_belongs_to_counter_after_save
     topic = Topic.create!(title: "monday night")
 
-    assert_queries(2) do
+    assert_queries_count(4) do
       topic.replies.create!(title: "re: monday night", content: "football")
     end
 
@@ -766,7 +792,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     debate.touch(time: time)
     debate2.touch(time: time)
 
-    assert_queries(3) do
+    assert_queries_count(5) do
       reply.parent_title = "debate"
       reply.save!
     end
@@ -777,7 +803,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     debate.touch(time: time)
     debate2.touch(time: time)
 
-    assert_queries(3) do
+    assert_queries_count(5) do
       reply.topic_with_primary_key = debate2
       reply.save!
     end
@@ -790,7 +816,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     line_item = LineItem.create!
     Invoice.create!(line_items: [line_item])
 
-    assert_queries(1) { line_item.touch }
+    assert_queries_count(3) { line_item.touch }
   end
 
   def test_belongs_to_with_touch_on_multiple_records
@@ -798,14 +824,14 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     line_item2 = LineItem.create!(amount: 2)
     Invoice.create!(line_items: [line_item, line_item2])
 
-    assert_queries(1) do
+    assert_queries_count(3) do
       LineItem.transaction do
         line_item.touch
         line_item2.touch
       end
     end
 
-    assert_queries(2) do
+    assert_queries_count(6) do
       line_item.touch
       line_item2.touch
     end
@@ -830,14 +856,14 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
 
     line_item.invoice = nil
 
-    assert_queries(2) { line_item.touch }
+    assert_queries_count(4) { line_item.touch }
   end
 
   def test_belongs_to_with_touch_option_on_update
     line_item = LineItem.create!
     Invoice.create!(line_items: [line_item])
 
-    assert_queries(2) { line_item.update amount: 10 }
+    assert_queries_count(4) { line_item.update amount: 10 }
   end
 
   def test_belongs_to_with_touch_option_on_empty_update
@@ -851,7 +877,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     line_item = LineItem.create!
     Invoice.create!(line_items: [line_item])
 
-    assert_queries(2) { line_item.destroy }
+    assert_queries_count(4) { line_item.destroy }
   end
 
   def test_belongs_to_with_touch_option_on_destroy_with_destroyed_parent
@@ -859,7 +885,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     invoice   = Invoice.create!(line_items: [line_item])
     invoice.destroy
 
-    assert_queries(1) { line_item.destroy }
+    assert_queries_count(3) { line_item.destroy }
   end
 
   def test_belongs_to_with_touch_option_on_touch_and_reassigned_parent
@@ -868,7 +894,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
 
     line_item.invoice = Invoice.create!
 
-    assert_queries(3) { line_item.touch }
+    assert_queries_count(5) { line_item.touch }
   end
 
   def test_belongs_to_counter_after_update
@@ -945,7 +971,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
   def test_dont_find_target_when_saving_foreign_key_after_stale_association_loaded
     client = Client.create!(name: "Test client", firm_with_basic_id: Firm.find(1))
     client.firm_id = Firm.create!(name: "Test firm").id
-    assert_queries(1) { client.save! }
+    assert_queries_count(3) { client.save! }
   end
 
   def test_field_name_same_as_foreign_key
@@ -1159,8 +1185,10 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
   end
 
   def test_belongs_to_proxy_should_respond_to_private_methods_via_send
-    companies(:first_firm).send(:private_method)
-    companies(:second_client).firm.send(:private_method)
+    assert_nothing_raised do
+      companies(:first_firm).send(:private_method)
+      companies(:second_client).firm.send(:private_method)
+    end
   end
 
   def test_save_of_record_with_loaded_belongs_to
@@ -1436,6 +1464,20 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_equal touch_time, car.reload.wheels_owned_at
   end
 
+  def test_polymorphic_stale_state_handles_nil_foreign_keys_correctly
+    klass = Class.new(ActiveRecord::Base) do
+      self.table_name = "records"
+
+      has_one :attachment, as: :record
+
+      def self.polymorphic_name
+        "Blob"
+      end
+    end
+
+    assert_nothing_raised { Attachment.create!(record: klass.build, record_type: "Document") }
+  end
+
   def test_build_with_conditions
     client = companies(:second_client)
     firm   = client.build_bob_firm
@@ -1649,12 +1691,12 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_not node.parent_previously_changed?
 
     node.parent = nodes(:grandparent)
-    assert node.parent_changed?
+    assert_predicate node, :parent_changed?
     assert_not node.parent_previously_changed?
 
     node.save!
     assert_not node.parent_changed?
-    assert node.parent_previously_changed?
+    assert_predicate node, :parent_previously_changed?
   end
 
   test "tracking change from persisted record to new record" do
@@ -1664,12 +1706,12 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_not node.parent_previously_changed?
 
     node.parent = Node.new(tree: node.tree, parent: nodes(:parent_a), name: "Child three")
-    assert node.parent_changed?
+    assert_predicate node, :parent_changed?
     assert_not node.parent_previously_changed?
 
     node.save!
     assert_not node.parent_changed?
-    assert node.parent_previously_changed?
+    assert_predicate node, :parent_previously_changed?
   end
 
   test "tracking change from persisted record to nil" do
@@ -1679,12 +1721,12 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_not node.parent_previously_changed?
 
     node.parent = nil
-    assert node.parent_changed?
+    assert_predicate node, :parent_changed?
     assert_not node.parent_previously_changed?
 
     node.save!
     assert_not node.parent_changed?
-    assert node.parent_previously_changed?
+    assert_predicate node, :parent_previously_changed?
   end
 
   test "tracking change from nil to persisted record" do
@@ -1694,12 +1736,12 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_not node.parent_previously_changed?
 
     node.parent = Node.create!(tree: node.tree, name: "Great-grandparent")
-    assert node.parent_changed?
+    assert_predicate node, :parent_changed?
     assert_not node.parent_previously_changed?
 
     node.save!
     assert_not node.parent_changed?
-    assert node.parent_previously_changed?
+    assert_predicate node, :parent_previously_changed?
   end
 
   test "tracking change from nil to new record" do
@@ -1709,12 +1751,12 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_not node.parent_previously_changed?
 
     node.parent = Node.new(tree: node.tree, name: "Great-grandparent")
-    assert node.parent_changed?
+    assert_predicate node, :parent_changed?
     assert_not node.parent_previously_changed?
 
     node.save!
     assert_not node.parent_changed?
-    assert node.parent_previously_changed?
+    assert_predicate node, :parent_previously_changed?
   end
 
   test "tracking polymorphic changes" do
@@ -1724,20 +1766,20 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     assert_not comment.author_previously_changed?
 
     comment.author = authors(:david)
-    assert comment.author_changed?
+    assert_predicate comment, :author_changed?
 
     comment.save!
     assert_not comment.author_changed?
-    assert comment.author_previously_changed?
+    assert_predicate comment, :author_previously_changed?
 
     assert_equal authors(:david).id, companies(:first_firm).id
 
     comment.author = companies(:first_firm)
-    assert comment.author_changed?
+    assert_predicate comment, :author_changed?
 
     comment.save!
     assert_not comment.author_changed?
-    assert comment.author_previously_changed?
+    assert_predicate comment, :author_previously_changed?
   end
 
   class ShipRequired < ActiveRecord::Base
@@ -1752,14 +1794,14 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     ship = ShipRequired.create!(name: "Medusa", developer: david)
     assert_equal david, ship.developer
 
-    assert_queries(2) do # UPDATE and SELECT to check developer presence
+    assert_queries_count(4) do # UPDATE and SELECT to check developer presence
       ship.update!(developer_id: jamis.id)
     end
 
     ship.update_column(:developer_id, nil)
     ship.reload
 
-    assert_queries(2) do # UPDATE and SELECT to check developer presence
+    assert_queries_count(4) do # UPDATE and SELECT to check developer presence
       ship.update!(developer_id: david.id)
     end
   end
@@ -1769,7 +1811,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     ship = ShipRequired.create!(name: "Medusa", developer: david)
     ship.reload # unload developer association
 
-    assert_queries(1) do # UPDATE only, no SELECT to check developer presence
+    assert_queries_count(3) do # UPDATE only, no SELECT to check developer presence
       ship.update!(name: "Leviathan")
     end
   end
@@ -1788,7 +1830,7 @@ class BelongsToAssociationsTest < ActiveRecord::TestCase
     ship = model.create!(name: "Medusa", developer: david)
     ship.reload # unload developer association
 
-    assert_queries(2) do # UPDATE and SELECT to check developer presence
+    assert_queries_count(4) do # UPDATE and SELECT to check developer presence
       ship.update!(name: "Leviathan")
     end
   ensure
