@@ -1,6 +1,7 @@
 # frozen_string_literal: true
 
 require "cases/helper"
+require "models/auto_id"
 require "models/aircraft"
 require "models/dashboard"
 require "models/clothing_item"
@@ -17,6 +18,7 @@ require "models/project"
 require "models/minimalistic"
 require "models/parrot"
 require "models/minivan"
+require "models/car"
 require "models/person"
 require "models/ship"
 require "models/admin"
@@ -29,7 +31,7 @@ require "models/pk_autopopulated_by_a_trigger_record"
 
 class PersistenceTest < ActiveRecord::TestCase
   fixtures :topics, :companies, :developers, :accounts, :minimalistics, :authors, :author_addresses,
-    :posts, :minivans, :clothing_items, :cpk_books
+    :posts, :minivans, :clothing_items, :cpk_books, :people, :cars
 
   def test_populates_non_primary_key_autoincremented_column
     topic = TitlePrimaryKeyTopic.create!(title: "title pk topic")
@@ -37,6 +39,22 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_not_nil topic.attributes["id"]
   end
 
+  def test_populates_autoincremented_id_pk_regardless_of_its_position_in_columns_list
+    auto_populated_column_names = AutoId.columns.select(&:auto_populated?).map(&:name)
+
+    # It's important we test a scenario where tables has more than one auto populated column
+    # and the first column is not the primary key. Otherwise it will be a regular test not asserting this special case.
+    assert auto_populated_column_names.size > 1
+    assert_not_equal AutoId.primary_key, auto_populated_column_names.first
+
+    record = AutoId.create!
+    last_id = AutoId.last.id
+
+    assert_not_nil last_id
+    assert last_id > 0
+    assert_equal last_id, record.id
+  end
+
   def test_populates_non_primary_key_autoincremented_column_for_a_cpk_model
     order = Cpk::Order.create(shop_id: 111_222)
 
@@ -45,31 +63,56 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_not_nil order_id
   end
 
-  def test_fills_auto_populated_columns_on_creation
-    record_with_defaults = Default.create
-    assert_not_nil record_with_defaults.id
-    assert_equal "Ruby on Rails", record_with_defaults.ruby_on_rails
+  if current_adapter?(:PostgreSQLAdapter)
+    def test_fills_auto_populated_columns_on_creation
+      record = Default.create
+      assert_not_nil record.id
+      assert_equal "Ruby on Rails", record.ruby_on_rails
 
-    if current_adapter?(:PostgreSQLAdapter) && ActiveRecord::Base.connection.supports_virtual_columns?
-      assert_not_nil record_with_defaults.virtual_stored_number
-    end
-
-    assert_not_nil record_with_defaults.random_number
-    assert_not_nil record_with_defaults.modified_date
-    assert_not_nil record_with_defaults.modified_date_function
-    assert_not_nil record_with_defaults.modified_time
-    assert_not_nil record_with_defaults.modified_time_without_precision
-    assert_not_nil record_with_defaults.modified_time_function
-
-    if current_adapter?(:PostgreSQLAdapter) && ActiveRecord::Base.connection.supports_identity_columns?
-      klass = Class.new(ActiveRecord::Base) do
-        self.table_name = "postgresql_identity_table"
+      if supports_virtual_columns?
+        assert_not_nil record.virtual_stored_number
       end
 
-      record = klass.create!
-      assert_not_nil record.id
+      assert_not_nil record.random_number
+      assert_not_nil record.modified_date
+      assert_not_nil record.modified_date_function
+      assert_not_nil record.modified_time
+      assert_not_nil record.modified_time_without_precision
+      assert_not_nil record.modified_time_function
+
+      if supports_identity_columns?
+        klass = Class.new(ActiveRecord::Base) do
+          self.table_name = "postgresql_identity_table"
+        end
+
+        record = klass.create!
+        assert_not_nil record.id
+      end
     end
-  end if current_adapter?(:PostgreSQLAdapter) || current_adapter?(:SQLite3Adapter)
+  elsif current_adapter?(:SQLite3Adapter)
+    def test_fills_auto_populated_columns_on_creation
+      record = Default.create
+      assert_not_nil record.id
+      assert_equal "Ruby on Rails", record.ruby_on_rails
+
+      assert_not_nil record.random_number
+      assert_not_nil record.modified_date
+      assert_not_nil record.modified_date_function
+      assert_not_nil record.modified_time
+      assert_not_nil record.modified_time_without_precision
+      assert_not_nil record.modified_time_function
+    end
+  elsif current_adapter?(:Mysql2Adapter, :TrilogyAdapter)
+    def test_fills_auto_populated_columns_on_creation
+      record = Default.create
+      assert_not_nil record.id
+      assert_not_nil record.char1
+
+      if supports_default_expression? && supports_insert_returning?
+        assert_not_nil record.uuid
+      end
+    end
+  end
 
   def test_update_many
     topic_data = { 1 => { "content" => "1 updated" }, 2 => { "content" => "2 updated" } }
@@ -719,7 +762,7 @@ class PersistenceTest < ActiveRecord::TestCase
 
   def test_becomes_default_sti_subclass
     original_type = Topic.columns_hash["type"].default
-    ActiveRecord::Base.connection.change_column_default :topics, :type, "Reply"
+    ActiveRecord::Base.lease_connection.change_column_default :topics, :type, "Reply"
     Topic.reset_column_information
 
     reply = topics(:second)
@@ -729,7 +772,7 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_instance_of Topic, topic
 
   ensure
-    ActiveRecord::Base.connection.change_column_default :topics, :type, original_type
+    ActiveRecord::Base.lease_connection.change_column_default :topics, :type, original_type
     Topic.reset_column_information
   end
 
@@ -811,7 +854,7 @@ class PersistenceTest < ActiveRecord::TestCase
   def test_delete
     topic = Topic.find(1)
     assert_equal topic, topic.delete, "topic.delete did not return self"
-    assert topic.frozen?, "topic not frozen after delete"
+    assert_predicate topic, :frozen?, "topic not frozen after delete"
     assert_raise(ActiveRecord::RecordNotFound) { Topic.find(topic.id) }
   end
 
@@ -830,14 +873,14 @@ class PersistenceTest < ActiveRecord::TestCase
   def test_destroy
     topic = Topic.find(1)
     assert_equal topic, topic.destroy, "topic.destroy did not return self"
-    assert topic.frozen?, "topic not frozen after destroy"
+    assert_predicate topic, :frozen?, "topic not frozen after destroy"
     assert_raise(ActiveRecord::RecordNotFound) { Topic.find(topic.id) }
   end
 
   def test_destroy!
     topic = Topic.find(1)
     assert_equal topic, topic.destroy!, "topic.destroy! did not return self"
-    assert topic.frozen?, "topic not frozen after destroy!"
+    assert_predicate topic, :frozen?, "topic not frozen after destroy!"
     assert_raise(ActiveRecord::RecordNotFound) { Topic.find(topic.id) }
   end
 
@@ -883,6 +926,16 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_nil Topic.find(2).last_read
   end
 
+  def test_update_all_with_custom_sql_as_value
+    person = people(:michael)
+    person.update!(cars_count: 0)
+
+    Person.update_all(cars_count: Arel.sql(<<~SQL))
+      select count(*) from cars where cars.person_id = people.id
+    SQL
+    assert_equal 1, person.reload.cars_count
+  end
+
   def test_delete_new_record
     client = Client.new(name: "37signals")
     client.delete
@@ -1117,8 +1170,8 @@ class PersistenceTest < ActiveRecord::TestCase
     t.update_column(:title, "super_title")
     assert_equal "John", t.author_name
     assert_equal "super_title", t.title
-    assert t.changed?, "topic should have changed"
-    assert t.author_name_changed?, "author_name should have changed"
+    assert_predicate t, :changed?, "topic should have changed"
+    assert_predicate t, :author_name_changed?, "author_name should have changed"
 
     t.reload
     assert_equal author_name, t.author_name
@@ -1216,8 +1269,8 @@ class PersistenceTest < ActiveRecord::TestCase
     t.update_columns(title: "super_title")
     assert_equal "John", t.author_name
     assert_equal "super_title", t.title
-    assert t.changed?, "topic should have changed"
-    assert t.author_name_changed?, "author_name should have changed"
+    assert_predicate t, :changed?, "topic should have changed"
+    assert_predicate t, :author_name_changed?, "author_name should have changed"
 
     t.reload
     assert_equal author_name, t.author_name
@@ -1364,6 +1417,17 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_nothing_raised { Reply.find(should_not_be_destroyed_reply.id) }
   end
 
+  def test_class_level_delete_with_invalid_ids
+    assert_no_queries do
+      assert_equal 0, Topic.delete(nil)
+      assert_equal 0, Topic.delete([])
+    end
+
+    assert_difference -> { Topic.count }, -1 do
+      assert_equal 1, Topic.delete(topics(:first).id)
+    end
+  end
+
   def test_class_level_delete_is_affected_by_scoping
     should_not_be_destroyed_reply = Reply.create("title" => "hello", "content" => "world")
     Topic.find(1).replies << should_not_be_destroyed_reply
@@ -1427,9 +1491,9 @@ class PersistenceTest < ActiveRecord::TestCase
   end
 
   def test_reload_via_querycache
-    ActiveRecord::Base.connection.enable_query_cache!
-    ActiveRecord::Base.connection.clear_query_cache
-    assert ActiveRecord::Base.connection.query_cache_enabled, "cache should be on"
+    ActiveRecord::Base.lease_connection.enable_query_cache!
+    ActiveRecord::Base.lease_connection.clear_query_cache
+    assert ActiveRecord::Base.lease_connection.query_cache_enabled, "cache should be on"
     parrot = Parrot.create(name: "Shane")
 
     # populate the cache with the SELECT result
@@ -1437,7 +1501,7 @@ class PersistenceTest < ActiveRecord::TestCase
     assert_equal parrot.id, found_parrot.id
 
     # Manually update the 'name' attribute in the DB directly
-    assert_equal 1, ActiveRecord::Base.connection.query_cache.length
+    assert_equal 1, ActiveRecord::Base.lease_connection.query_cache.size
     ActiveRecord::Base.uncached do
       found_parrot.name = "Mary"
       found_parrot.save
@@ -1450,7 +1514,7 @@ class PersistenceTest < ActiveRecord::TestCase
     found_parrot = Parrot.find(parrot.id)
     assert_equal "Mary", found_parrot.name
   ensure
-    ActiveRecord::Base.connection.disable_query_cache!
+    ActiveRecord::Base.lease_connection.disable_query_cache!
   end
 
   def test_save_touch_false
@@ -1472,7 +1536,7 @@ class PersistenceTest < ActiveRecord::TestCase
     child_class = Class.new(Topic)
     child_class.new # force schema to load
 
-    ActiveRecord::Base.connection.add_column(:topics, :foo, :string)
+    ActiveRecord::Base.lease_connection.add_column(:topics, :foo, :string)
     Topic.reset_column_information
 
     # this should redefine attribute methods
@@ -1482,13 +1546,13 @@ class PersistenceTest < ActiveRecord::TestCase
     assert child_class.instance_methods.include?(:foo_changed?)
     assert_equal "bar", child_class.new(foo: :bar).foo
   ensure
-    ActiveRecord::Base.connection.remove_column(:topics, :foo)
+    ActiveRecord::Base.lease_connection.remove_column(:topics, :foo)
     Topic.reset_column_information
   end
 
   def test_update_uses_query_constraints_config
     clothing_item = clothing_items(:green_t_shirt)
-    sql = capture_sql { clothing_item.update(description: "Lovely green t-shirt")  }.first
+    sql = capture_sql { clothing_item.update(description: "Lovely green t-shirt")  }.second
     assert_match(/WHERE .*clothing_type/, sql)
     assert_match(/WHERE .*color/, sql)
   end
@@ -1496,7 +1560,7 @@ class PersistenceTest < ActiveRecord::TestCase
   def test_save_uses_query_constraints_config
     clothing_item = clothing_items(:green_t_shirt)
     clothing_item.description = "Lovely green t-shirt"
-    sql = capture_sql { clothing_item.save }.first
+    sql = capture_sql { clothing_item.save }.second
     assert_match(/WHERE .*clothing_type/, sql)
     assert_match(/WHERE .*color/, sql)
   end
@@ -1510,7 +1574,7 @@ class PersistenceTest < ActiveRecord::TestCase
 
   def test_destroy_uses_query_constraints_config
     clothing_item = clothing_items(:green_t_shirt)
-    sql = capture_sql { clothing_item.destroy }.first
+    sql = capture_sql { clothing_item.destroy }.second
     assert_match(/WHERE .*clothing_type/, sql)
     assert_match(/WHERE .*color/, sql)
   end
@@ -1524,7 +1588,7 @@ class PersistenceTest < ActiveRecord::TestCase
 
   def test_update_attribute_uses_query_constraints_config
     clothing_item = clothing_items(:green_t_shirt)
-    sql = capture_sql { clothing_item.update_attribute(:description, "Lovely green t-shirt") }.first
+    sql = capture_sql { clothing_item.update_attribute(:description, "Lovely green t-shirt") }.second
     assert_match(/WHERE .*clothing_type/, sql)
     assert_match(/WHERE .*color/, sql)
   end
@@ -1533,7 +1597,7 @@ class PersistenceTest < ActiveRecord::TestCase
     clothing_item = clothing_items(:green_t_shirt)
     clothing_item.color = "blue"
     clothing_item.description = "Now it's a blue t-shirt"
-    sql = capture_sql { clothing_item.save }.first
+    sql = capture_sql { clothing_item.save }.second
     assert_match(/WHERE .*clothing_type/, sql)
     assert_match(/WHERE .*color/, sql)
 
@@ -1545,7 +1609,7 @@ class PersistenceTest < ActiveRecord::TestCase
 
     assert_not_nil record.id
     assert record.id > 0
-  end if current_adapter?(:PostgreSQLAdapter)
+  end if supports_insert_returning? && !current_adapter?(:SQLite3Adapter)
 end
 
 class QueryConstraintsTest < ActiveRecord::TestCase
