@@ -36,18 +36,14 @@ class StrictLoadingTest < ActiveRecord::TestCase
     end
 
     assert developer.strict_loading!(mode: :n_plus_one_only)
-    assert developer.strict_loading_n_plus_one_only?
+    assert_predicate developer, :strict_loading_n_plus_one_only?
   end
 
-  def test_strict_loading_n_plus_one_only_mode
+  def test_strict_loading_n_plus_one_only_mode_with_has_many
     developer = Developer.first
-    ship = Ship.first
-    ShipPart.create!(name: "Stern", ship: ship)
     firm = Firm.create!(name: "NASA")
-    project = Project.create!(name: "Apollo", firm: firm)
+    developer.projects << Project.create!(name: "Apollo", firm: firm)
 
-    ship.update_column(:developer_id, developer.id)
-    developer.projects << project
     developer.reload
 
     developer.strict_loading!(mode: :n_plus_one_only)
@@ -63,6 +59,18 @@ class StrictLoadingTest < ActiveRecord::TestCase
     assert_raises ActiveRecord::StrictLoadingViolationError do
       developer.projects.last.firm
     end
+  end
+
+  def test_strict_loading_n_plus_one_only_mode_with_belongs_to
+    developer = Developer.first
+    ship = Ship.first
+    ShipPart.create!(name: "Stern", ship: ship)
+
+    ship.update_column(:developer_id, developer.id)
+    developer.reload
+
+    developer.strict_loading!(mode: :n_plus_one_only)
+    assert_predicate developer, :strict_loading?
 
     # Does not raise when a belongs_to association (:ship) loads its
     # has_many association (:parts)
@@ -92,12 +100,12 @@ class StrictLoadingTest < ActiveRecord::TestCase
 
   def test_strict_loading
     Developer.all.each { |d| assert_not d.strict_loading? }
-    Developer.strict_loading.each { |d| assert d.strict_loading? }
+    Developer.strict_loading.each { |d| assert_predicate d, :strict_loading? }
   end
 
   def test_strict_loading_by_default
     with_strict_loading_by_default(Developer) do
-      Developer.all.each { |d| assert d.strict_loading? }
+      Developer.all.each { |d| assert_predicate d, :strict_loading? }
       Developer.strict_loading(false).each { |d| assert_not d.strict_loading? }
     end
   end
@@ -605,6 +613,14 @@ class StrictLoadingTest < ActiveRecord::TestCase
     end
   end
 
+  def test_does_not_raise_when_checking_if_new_record_included_in_eager_loaded_habtm_relation
+    Developer.first.projects << Project.first
+
+    developer = Developer.includes(:projects).strict_loading.first
+
+    assert_nothing_raised { developer.projects.include?(Project.new) }
+  end
+
   def test_strict_loading_violation_raises_by_default
     assert_equal :raise, ActiveRecord.action_on_strict_loading_violation
 
