@@ -71,9 +71,9 @@ module ActiveRecord
 
       def test_raises_an_error_when_called_with_protected_environment
         protected_environments = ActiveRecord::Base.protected_environments
-        current_env            = ActiveRecord::Base.connection.migration_context.current_environment
+        current_env            = ActiveRecord::Base.connection_pool.migration_context.current_environment
 
-        ActiveRecord::Base.connection.internal_metadata[:environment] = current_env
+        ActiveRecord::Base.connection_pool.internal_metadata[:environment] = current_env
 
         assert_called_on_instance_of(
           ActiveRecord::MigrationContext,
@@ -97,9 +97,9 @@ module ActiveRecord
 
       def test_raises_an_error_when_called_with_protected_environment_which_name_is_a_symbol
         protected_environments = ActiveRecord::Base.protected_environments
-        current_env            = ActiveRecord::Base.connection.migration_context.current_environment
+        current_env            = ActiveRecord::Base.connection_pool.migration_context.current_environment
 
-        ActiveRecord::Base.connection.internal_metadata[:environment] = current_env
+        ActiveRecord::Base.connection_pool.internal_metadata[:environment] = current_env
 
         assert_called_on_instance_of(
           ActiveRecord::MigrationContext,
@@ -121,9 +121,9 @@ module ActiveRecord
       end
 
       def test_raises_an_error_if_no_migrations_have_been_made
-        connection = ActiveRecord::Base.connection
-        internal_metadata = connection.internal_metadata
-        schema_migration = connection.schema_migration
+        pool = ActiveRecord::Base.connection_pool
+        internal_metadata = pool.internal_metadata
+        schema_migration = pool.schema_migration
         schema_migration.create_table
         schema_migration.create_version("1")
 
@@ -135,17 +135,18 @@ module ActiveRecord
           ActiveRecord::Tasks::DatabaseTasks.check_protected_environments!("arunit")
         end
       ensure
+        pool.automatic_reconnect = true
         schema_migration.delete_version("1")
         internal_metadata.create_table
       end
 
       private
       def recreate_metadata_tables
-        schema_migration = ActiveRecord::Base.connection.schema_migration
+        schema_migration = ActiveRecord::Base.connection_pool.schema_migration
         schema_migration.drop_table
         schema_migration.create_table
 
-        internal_metadata = ActiveRecord::Base.connection.internal_metadata
+        internal_metadata = ActiveRecord::Base.connection_pool.internal_metadata
         internal_metadata.drop_table
         internal_metadata.create_table
       end
@@ -161,22 +162,22 @@ module ActiveRecord
 
         with_multi_db_configurations(env) do
           protected_environments = ActiveRecord::Base.protected_environments
-          current_env = ActiveRecord::Base.connection.migration_context.current_environment
+          current_env = ActiveRecord::Base.connection_pool.migration_context.current_environment
           assert_equal current_env, env
 
           ActiveRecord::Base.establish_connection(:primary)
-          ActiveRecord::Base.connection.internal_metadata.create_table_and_set_flags(current_env)
+          ActiveRecord::Base.connection_pool.internal_metadata.create_table_and_set_flags(current_env)
 
           ActiveRecord::Base.establish_connection(:secondary)
-          ActiveRecord::Base.connection.internal_metadata.create_table_and_set_flags(current_env)
+          ActiveRecord::Base.connection_pool.internal_metadata.create_table_and_set_flags(current_env)
 
           assert_not_includes protected_environments, current_env
           # Assert not raises
           ActiveRecord::Tasks::DatabaseTasks.check_protected_environments!(env)
 
           ActiveRecord::Base.establish_connection(:secondary)
-          connection = ActiveRecord::Base.connection
-          schema_migration = connection.schema_migration
+          pool = ActiveRecord::Base.connection_pool
+          schema_migration = pool.schema_migration
           schema_migration.create_table
           schema_migration.create_version("1")
 
@@ -211,8 +212,8 @@ module ActiveRecord
         ensure
           [:primary, :secondary].each do |db|
             ActiveRecord::Base.establish_connection(db)
-            ActiveRecord::Base.connection.schema_migration.delete_all_versions
-            ActiveRecord::Base.connection.internal_metadata.delete_all_entries
+            ActiveRecord::Base.connection_pool.schema_migration.delete_all_versions
+            ActiveRecord::Base.connection_pool.internal_metadata.delete_all_entries
           end
           ActiveRecord::Base.configurations = old_configurations
           ActiveRecord::Base.establish_connection(:arunit)
@@ -221,6 +222,16 @@ module ActiveRecord
   end
 
   class DatabaseTasksRegisterTask < ActiveRecord::TestCase
+    setup do
+      @tasks_was = ActiveRecord::Tasks::DatabaseTasks.instance_variable_get(:@tasks).dup
+      @adapters_was = ActiveRecord::ConnectionAdapters.instance_variable_get(:@adapters).dup
+    end
+
+    teardown do
+      ActiveRecord::Tasks::DatabaseTasks.instance_variable_set(:@tasks, @tasks_was)
+      ActiveRecord::ConnectionAdapters.instance_variable_set(:@adapters, @adapters_was)
+    end
+
     def test_register_task
       klazz = Class.new do
         def initialize(*arguments); end
@@ -230,8 +241,8 @@ module ActiveRecord
 
       klazz.stub(:new, instance) do
         assert_called_with(instance, :structure_dump, ["awesome-file.sql", nil]) do
-          ActiveRecord::Tasks::DatabaseTasks.register_task(/foo/, klazz)
-          ActiveRecord::Tasks::DatabaseTasks.structure_dump({ "adapter" => :foo }, "awesome-file.sql")
+          ActiveRecord::Tasks::DatabaseTasks.register_task(/abstract/, klazz)
+          ActiveRecord::Tasks::DatabaseTasks.structure_dump({ "adapter" => "abstract" }, "awesome-file.sql")
         end
       end
     end
@@ -245,6 +256,7 @@ module ActiveRecord
 
       klazz.stub(:new, instance) do
         assert_called_with(instance, :structure_dump, ["awesome-file.sql", nil]) do
+          ActiveRecord::ConnectionAdapters.register("custom_mysql", "ActiveRecord::ConnectionAdapters::Mysql2Adapter", "active_record/connection_adapters/mysql2_adapter")
           ActiveRecord::Tasks::DatabaseTasks.register_task(/custom_mysql/, klazz)
           ActiveRecord::Tasks::DatabaseTasks.structure_dump({ "adapter" => :custom_mysql }, "awesome-file.sql")
         end
@@ -253,7 +265,7 @@ module ActiveRecord
 
     def test_unregistered_task
       assert_raise(ActiveRecord::Tasks::DatabaseNotSupported) do
-        ActiveRecord::Tasks::DatabaseTasks.structure_dump({ "adapter" => :bar }, "awesome-file.sql")
+        ActiveRecord::Tasks::DatabaseTasks.structure_dump({ "adapter" => "abstract" }, "awesome-file.sql")
       end
     end
   end
@@ -278,7 +290,7 @@ module ActiveRecord
         ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, dir) do
           path = File.join(dir, "schema_cache.yml")
           assert_not File.file?(path)
-          ActiveRecord::Tasks::DatabaseTasks.dump_schema_cache(ActiveRecord::Base.connection, path)
+          ActiveRecord::Tasks::DatabaseTasks.dump_schema_cache(ActiveRecord::Base.lease_connection, path)
           assert File.file?(path)
         end
       end
@@ -304,8 +316,38 @@ module ActiveRecord
       old_path = ENV["SCHEMA_CACHE"]
       ENV.delete("SCHEMA_CACHE")
 
+      config = DatabaseConfigurations::HashConfig.new("development", "primary", {})
+
       ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "db") do
-        path = ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename("primary")
+        path = ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename(config)
+        assert_equal "db/schema_cache.yml", path
+      end
+    ensure
+      ENV["SCHEMA_CACHE"] = old_path
+    end
+
+    def test_cache_dump_default_filename_with_custom_db_dir
+      old_path = ENV["SCHEMA_CACHE"]
+      ENV.delete("SCHEMA_CACHE")
+
+      config = DatabaseConfigurations::HashConfig.new("development", "primary", {})
+
+      ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "my_db") do
+        path = ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename(config)
+        assert_equal "my_db/schema_cache.yml", path
+      end
+    ensure
+      ENV["SCHEMA_CACHE"] = old_path
+    end
+
+    def test_deprecated_cache_dump_default_filename
+      old_path = ENV["SCHEMA_CACHE"]
+      ENV.delete("SCHEMA_CACHE")
+
+      ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "db") do
+        path = assert_deprecated(ActiveRecord.deprecator) do
+          ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename("primary")
+        end
         assert_equal "db/schema_cache.yml", path
       end
     ensure
@@ -316,8 +358,24 @@ module ActiveRecord
       old_path = ENV["SCHEMA_CACHE"]
       ENV.delete("SCHEMA_CACHE")
 
+      config = DatabaseConfigurations::HashConfig.new("development", "alternate", {})
+
       ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "db") do
-        path = ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename("alternate")
+        path = ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename(config)
+        assert_equal "db/alternate_schema_cache.yml", path
+      end
+    ensure
+      ENV["SCHEMA_CACHE"] = old_path
+    end
+
+    def test_deprecated_cache_dump_alternate_filename
+      old_path = ENV["SCHEMA_CACHE"]
+      ENV.delete("SCHEMA_CACHE")
+
+      ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "db") do
+        path = assert_deprecated(ActiveRecord.deprecator) do
+          ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename("alternate")
+        end
         assert_equal "db/alternate_schema_cache.yml", path
       end
     ensure
@@ -328,9 +386,27 @@ module ActiveRecord
       old_path = ENV["SCHEMA_CACHE"]
       ENV["SCHEMA_CACHE"] = "tmp/something.yml"
 
+      config = DatabaseConfigurations::HashConfig.new("development", "primary", {})
+
       ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "db") do
-        path = ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename("primary")
-        assert_equal "tmp/something.yml", path
+        path = assert_deprecated(/Setting `ENV\["SCHEMA_CACHE"\]` is deprecated and will be removed in Rails 8\.0\. Configure the `:schema_cache_path` in the database configuration instead\. \(/, ActiveRecord.deprecator) do
+          ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename(config)
+        end
+        assert_equal "db/schema_cache.yml", path
+      end
+    ensure
+      ENV["SCHEMA_CACHE"] = old_path
+    end
+
+    def test_deprecated_cache_dump_filename_with_env_override
+      old_path = ENV["SCHEMA_CACHE"]
+      ENV["SCHEMA_CACHE"] = "tmp/something.yml"
+
+      ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "db") do
+        path = assert_deprecated(/Passing a database name to `cache_dump_filename` is deprecated and will be removed in Rails 8\.0\. Pass a `ActiveRecord::DatabaseConfigurations::DatabaseConfig` object instead\. \(/, ActiveRecord.deprecator) do
+          ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename("primary")
+        end
+        assert_equal "db/schema_cache.yml", path
       end
     ensure
       ENV["SCHEMA_CACHE"] = old_path
@@ -340,8 +416,39 @@ module ActiveRecord
       old_path = ENV["SCHEMA_CACHE"]
       ENV.delete("SCHEMA_CACHE")
 
+      config = DatabaseConfigurations::HashConfig.new("development", "primary", { schema_cache_path:  "tmp/something.yml" })
+
       ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "db") do
-        path = ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename("primary", schema_cache_path: "tmp/something.yml")
+        path = ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename(config)
+        assert_equal "tmp/something.yml", path
+      end
+    ensure
+      ENV["SCHEMA_CACHE"] = old_path
+    end
+
+
+    def test_cache_dump_filename_with_path_from_the_argument_has_precedence
+      old_path = ENV["SCHEMA_CACHE"]
+      ENV.delete("SCHEMA_CACHE")
+
+      config = DatabaseConfigurations::HashConfig.new("development", "primary", { schema_cache_path:  "tmp/something.yml" })
+
+      ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "db") do
+        path = ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename(config, schema_cache_path: "tmp/another.yml")
+        assert_equal "tmp/another.yml", path
+      end
+    ensure
+      ENV["SCHEMA_CACHE"] = old_path
+    end
+
+    def test_deprecated_cache_dump_filename_with_path_from_the_argument
+      old_path = ENV["SCHEMA_CACHE"]
+      ENV.delete("SCHEMA_CACHE")
+
+      ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "db") do
+        path = assert_deprecated(ActiveRecord.deprecator) do
+          ActiveRecord::Tasks::DatabaseTasks.cache_dump_filename("primary", schema_cache_path: "tmp/something.yml")
+        end
         assert_equal "tmp/something.yml", path
       end
     ensure
@@ -391,7 +498,7 @@ module ActiveRecord
 
   class DatabaseTasksCreateAllTest < ActiveRecord::TestCase
     def setup
-      @configurations = { "development" => { "database" => "my-db" } }
+      @configurations = { "development" => { "adapter" => "abstract", "database" => "my-db" } }
 
       $stdout, @original_stdout = StringIO.new, $stdout
       $stderr, @original_stderr = StringIO.new, $stderr
@@ -469,9 +576,7 @@ module ActiveRecord
 
         # To refrain from connecting to a newly created empty DB in
         # sqlite3_mem tests
-        pool_mock = Minitest::Mock.new
-        pool_mock.expect(:connection, nil)
-        ActiveRecord::Base.connection_handler.stub(:establish_connection, pool_mock, &block)
+        ActiveRecord::Base.connection_handler.stub(:establish_connection, nil, &block)
       ensure
         ActiveRecord::Base.configurations = old_configurations
       end
@@ -482,8 +587,8 @@ module ActiveRecord
 
     def setup
       @configurations = {
-        "development" => { "database" => "dev-db" },
-        "test"        => { "database" => "test-db" },
+        "development" => { "adapter" => "abstract", "database" => "dev-db" },
+        "test"        => { "adapter" => "abstract", "database" => "test-db" },
         "production"  => { "url" => "abstract://prod-db-host/prod-db" }
       }
     end
@@ -605,9 +710,17 @@ module ActiveRecord
 
     def setup
       @configurations = {
-        "development" => { "primary" => { "database" => "dev-db" }, "secondary" => { "database" => "secondary-dev-db" } },
-        "test" => { "primary" => { "database" => "test-db" }, "secondary" => { "database" => "secondary-test-db" } },
-        "production" => { "primary" => { "url" => "abstract://prod-db-host/prod-db" }, "secondary" => { "url" => "abstract://secondary-prod-db-host/secondary-prod-db" } }
+        "development" => {
+          "primary" => { "adapter" => "abstract", "database" => "dev-db" },
+          "secondary" => { "adapter" => "abstract", "database" => "secondary-dev-db" },
+        },
+        "test" => {
+          "primary" => { "adapter" => "abstract", "database" => "test-db" },
+          "secondary" => { "adapter" => "abstract", "database" => "secondary-test-db" },
+        },
+        "production" => {
+          "primary" => { "url" => "abstract://prod-db-host/prod-db" },
+          "secondary" => { "url" => "abstract://secondary-prod-db-host/secondary-prod-db" } }
       }
     end
 
@@ -729,7 +842,7 @@ module ActiveRecord
 
   class DatabaseTasksDropAllTest < ActiveRecord::TestCase
     def setup
-      @configurations = { development: { "database" => "my-db" } }
+      @configurations = { development: { "adapter" => "abstract", "database" => "my-db" } }
 
       $stdout, @original_stdout = StringIO.new, $stdout
       $stderr, @original_stderr = StringIO.new, $stderr
@@ -816,8 +929,8 @@ module ActiveRecord
 
     def setup
       @configurations = {
-        "development" => { "database" => "dev-db" },
-        "test"        => { "database" => "test-db" },
+        "development" => { "adapter" => "abstract", "database" => "dev-db" },
+        "test"        => { "adapter" => "abstract", "database" => "test-db" },
         "production"  => { "url" => "abstract://prod-db-host/prod-db" }
       }
     end
@@ -907,9 +1020,18 @@ module ActiveRecord
 
     def setup
       @configurations = {
-        "development" => { "primary" => { "database" => "dev-db" }, "secondary" => { "database" => "secondary-dev-db" } },
-        "test" => { "primary" => { "database" => "test-db" }, "secondary" => { "database" => "secondary-test-db" } },
-        "production" => { "primary" => { "url" => "abstract://prod-db-host/prod-db" }, "secondary" => { "url" => "abstract://secondary-prod-db-host/secondary-prod-db" } }
+        "development" => {
+          "primary" => { "adapter" => "abstract", "database" => "dev-db" },
+          "secondary" => { "adapter" => "abstract", "database" => "secondary-dev-db" },
+        },
+        "test" => {
+          "primary" => { "adapter" => "abstract", "database" => "test-db" },
+          "secondary" => { "adapter" => "abstract", "database" => "secondary-test-db" },
+        },
+        "production" => {
+          "primary" => { "url" => "abstract://prod-db-host/prod-db" },
+          "secondary" => { "url" => "abstract://secondary-prod-db-host/secondary-prod-db" },
+        },
       }
     end
 
@@ -1009,11 +1131,11 @@ module ActiveRecord
       # Use a memory db here to avoid having to rollback at the end
       setup do
         migrations_path = [MIGRATIONS_ROOT, folder_name].join("/")
-        file = ActiveRecord::Base.connection.raw_connection.filename
+        file = ActiveRecord::Base.lease_connection.raw_connection.filename
         @conn = ActiveRecord::Base.establish_connection adapter: "sqlite3",
           database: ":memory:", migrations_paths: migrations_path
         source_db = SQLite3::Database.new file
-        dest_db = ActiveRecord::Base.connection.raw_connection
+        dest_db = ActiveRecord::Base.lease_connection.raw_connection
         backup = SQLite3::Backup.new(dest_db, "main", source_db, "main")
         backup.step(-1)
         backup.finish
@@ -1138,7 +1260,7 @@ module ActiveRecord
   class DatabaseTasksMigrateStatusTest < DatabaseTasksMigrationTestCase
     if current_adapter?(:SQLite3Adapter) && !in_memory_db?
       def setup
-        @schema_migration = ActiveRecord::Base.connection.schema_migration
+        @schema_migration = ActiveRecord::Base.connection_pool.schema_migration
       end
 
       def test_migrate_status_table
@@ -1209,7 +1331,7 @@ module ActiveRecord
     end
 
     def test_migrate_clears_schema_cache_afterward
-      assert_called(ActiveRecord::Base.connection.schema_cache, :clear!) do
+      assert_called(ActiveRecord::Base.schema_cache, :clear!) do
         ActiveRecord::Tasks::DatabaseTasks.migrate
       end
     end
@@ -1233,9 +1355,9 @@ module ActiveRecord
     def test_purges_current_environment_database
       old_configurations = ActiveRecord::Base.configurations
       configurations = {
-        "development" => { "database" => "dev-db" },
-        "test"        => { "database" => "test-db" },
-        "production"  => { "database" => "prod-db" }
+        "development" => { "adapter" => "abstract", "database" => "dev-db" },
+        "test"        => { "adapter" => "abstract", "database" => "test-db" },
+        "production"  => { "adapter" => "abstract", "database" => "prod-db" },
       }
 
       ActiveRecord::Base.configurations = configurations
@@ -1257,7 +1379,7 @@ module ActiveRecord
   class DatabaseTasksPurgeAllTest < ActiveRecord::TestCase
     def test_purge_all_local_configurations
       old_configurations = ActiveRecord::Base.configurations
-      configurations = { development: { "database" => "my-db" } }
+      configurations = { development: { "adapter" => "abstract", "database" => "my-db" } }
       ActiveRecord::Base.configurations = configurations
 
       assert_called_with(
@@ -1279,12 +1401,12 @@ module ActiveRecord
       fixtures :courses, :colleges
 
       def setup
-        connection = ARUnit2Model.connection
-        @schema_migration = connection.schema_migration
+        pool = ARUnit2Model.connection_pool
+        @schema_migration = pool.schema_migration
         @schema_migration.create_table
         @schema_migration.create_version(@schema_migration.table_name)
 
-        @internal_metadata = connection.internal_metadata
+        @internal_metadata = pool.internal_metadata
         @internal_metadata.create_table
         @internal_metadata[@internal_metadata.table_name] = nil
 
@@ -1347,9 +1469,18 @@ module ActiveRecord
 
     def setup
       @configurations = {
-        "development" => { "primary" => { "database" => "dev-db" }, "secondary" => { "database" => "secondary-dev-db" } },
-        "test" => { "primary" => { "database" => "test-db" }, "secondary" => { "database" => "secondary-test-db" } },
-        "production" => { "primary" => { "url" => "abstract://prod-db-host/prod-db" }, "secondary" => { "url" => "abstract://secondary-prod-db-host/secondary-prod-db" } }
+        "development" => {
+          "primary" => { "adapter" => "abstract", "database" => "dev-db" },
+          "secondary" => { "adapter" => "abstract", "database" => "secondary-dev-db" },
+        },
+        "test" => {
+          "primary" => { "adapter" => "abstract", "database" => "test-db" },
+          "secondary" => { "adapter" => "abstract", "database" => "secondary-test-db" },
+        },
+        "production" => {
+          "primary" => { "url" => "abstract://prod-db-host/prod-db" },
+          "secondary" => { "url" => "abstract://secondary-prod-db-host/secondary-prod-db" },
+         }
       }
     end
 
@@ -1453,7 +1584,7 @@ module ActiveRecord
     def test_charset_current
       old_configurations = ActiveRecord::Base.configurations
       configurations = {
-        "production" => { "database" => "prod-db" }
+        "production" => { "adapter" => "abstract", "database" => "prod-db" }
       }
 
       ActiveRecord::Base.configurations = configurations
@@ -1486,7 +1617,7 @@ module ActiveRecord
     def test_collation_current
       old_configurations = ActiveRecord::Base.configurations
       configurations = {
-        "production" => { "database" => "prod-db" }
+        "production" => { "adapter" => "abstract", "database" => "prod-db" }
       }
 
       ActiveRecord::Base.configurations = configurations
@@ -1656,7 +1787,7 @@ module ActiveRecord
 
   class DatabaseTasksCheckSchemaFileMethods < ActiveRecord::TestCase
     setup do
-      @configurations = { "development" => { "database" => "my-db" } }
+      @configurations = { "development" => { "adapter" => "abstract", "database" => "my-db" } }
     end
 
     def test_check_dump_filename_defaults
@@ -1692,7 +1823,10 @@ module ActiveRecord
     def test_check_dump_filename_defaults_for_non_primary_databases
       ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "/tmp") do
         configurations = {
-          "development" => { "primary" => { "database" => "dev-db" }, "secondary" => { "database" => "secondary-dev-db" } },
+          "development" => {
+            "primary" => { "adapter" => "abstract", "database" => "dev-db" },
+            "secondary" => { "adapter" => "abstract", "database" => "secondary-dev-db" },
+          },
         }
         with_stubbed_configurations(configurations) do
           assert_equal "/tmp/secondary_schema.rb", ActiveRecord::Tasks::DatabaseTasks.schema_dump_path(config_for("development", "secondary"))
@@ -1703,7 +1837,7 @@ module ActiveRecord
     def test_setting_schema_dump_to_nil
       ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "/tmp") do
         configurations = {
-          "development" => { "primary" => { "database" => "dev-db", "schema_dump" => false } },
+          "development" => { "primary" => { "adapter" => "abstract", "database" => "dev-db", "schema_dump" => false } },
         }
         with_stubbed_configurations(configurations) do
           assert_nil ActiveRecord::Tasks::DatabaseTasks.schema_dump_path(config_for("development", "primary"))
@@ -1716,7 +1850,10 @@ module ActiveRecord
       ENV["SCHEMA"] = "schema_path"
       ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "/tmp") do
         configurations = {
-          "development" => { "primary" => { "database" => "dev-db" }, "secondary" => { "database" => "secondary-dev-db" } },
+          "development" => {
+            "primary" => { "adapter" => "abstract", "database" => "dev-db" },
+            "secondary" => { "adapter" => "abstract", "database" => "secondary-dev-db" },
+          },
         }
         with_stubbed_configurations(configurations) do
           assert_equal "schema_path", ActiveRecord::Tasks::DatabaseTasks.schema_dump_path(config_for("development", "secondary"))
@@ -1730,7 +1867,10 @@ module ActiveRecord
       define_method("test_check_dump_filename_for_#{fmt}_format_with_non_primary_databases") do
         ActiveRecord::Tasks::DatabaseTasks.stub(:db_dir, "/tmp") do
           configurations = {
-            "development" => { "primary" => { "database" => "dev-db" }, "secondary" => { "database" => "secondary-dev-db" } },
+            "development" => {
+              "primary" => { "adapter" => "abstract", "database" => "dev-db" },
+              "secondary" => { "adapter" => "abstract", "database" => "secondary-dev-db" },
+            },
           }
           with_stubbed_configurations(configurations) do
             assert_equal "/tmp/secondary_#{filename}", ActiveRecord::Tasks::DatabaseTasks.schema_dump_path(config_for("development", "secondary"), fmt)
