@@ -1,6 +1,6 @@
 # frozen_string_literal: true
 
-require "active_support/testing/strict_warnings"
+require_relative "../../tools/strict_warnings"
 
 ORIG_ARGV = ARGV.dup
 
@@ -43,12 +43,6 @@ class ActiveSupport::TestCase
 
   include ActiveSupport::Testing::MethodCallAssertions
   include ActiveSupport::Testing::ErrorReporterAssertions
-
-  private
-    # Skips the current run on JRuby using Minitest::Assertions#skip
-    def jruby_skip(message = "")
-      skip message if defined?(JRUBY_VERSION)
-    end
 end
 
 require_relative "../../tools/test_common"
