@@ -5,13 +5,11 @@ require "bundler/inline"
 gemfile(true) do
   source "https://rubygems.org"
 
-  git_source(:github) { |repo| "https://github.com/#{repo}.git" }
-
   gem "rails"
   # If you want to test against edge Rails replace the previous line with this:
   # gem "rails", github: "rails/rails", branch: "main"
 
-  gem "sqlite3", "~> 1.4"
+  gem "sqlite3"
 end
 
 require "active_record/railtie"
@@ -20,6 +18,8 @@ require "action_mailbox/engine"
 require "tmpdir"
 
 class TestApp < Rails::Application
+  config.load_defaults Rails::VERSION::STRING.to_f
+
   config.root = __dir__
   config.hosts << "example.org"
   config.eager_load = false
