@@ -25,13 +25,6 @@ module ApplicationTests
       end
     end
 
-    def with_env(env)
-      env.each { |k, v| ENV[k.to_s] = v }
-      yield
-    ensure
-      env.each_key { |k| ENV.delete k.to_s }
-    end
-
     def clean_assets!
       quietly do
         rails ["assets:clobber"]
@@ -41,7 +34,7 @@ module ApplicationTests
     def assert_file_exists(filename)
       globbed = Dir[filename]
       assert Dir.exist?(File.dirname(filename)), "Directory #{File.dirname(filename)} does not exist"
-      assert globbed.one?, "Found #{globbed.size} files matching #{filename}. All files in the directory: #{Dir.entries(File.dirname(filename)).inspect}"
+      assert_predicate globbed, :one?, "Found #{globbed.size} files matching #{filename}. All files in the directory: #{Dir.entries(File.dirname(filename)).inspect}"
     end
 
     def assert_no_file_exists(filename)
@@ -250,7 +243,6 @@ module ApplicationTests
     test "assets do not require any assets group gem when manifest file is present" do
       app_file "app/assets/javascripts/application.js", "alert();"
       app_file "app/assets/config/manifest.js", "//= link application.js"
-      add_to_env_config "production", "config.public_file_server.enabled = true"
 
       precompile! RAILS_ENV: "production"
 
