@@ -187,7 +187,7 @@ class Releaser < Rake::TaskLib
   end
 
   def npm_tag
-    pre_release? ? "pre" : "7-1-stable"
+    pre_release? ? "pre" : "latest"
   end
 
   # This "npm-ifies" the current version number
